#include "runtime.h"

#include <string.h>

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif

VY_DLLEXPORT int vyCompareSpanToString(vy_text_span span, const char *cmp) {
    size_t cmp_len = strlen(cmp);
    if (cmp_len != (size_t)span.length)
        return (span.length < cmp_len) ? -1 : 1;
    for (size_t i = 0; i < cmp_len; ++i) {
        if (span.start[i] != cmp[i])
            return span.start[i] - cmp[i];
    }
    return 0;
}

VY_DLLEXPORT vy_result vyUTF8ToWStr(const char *utf8, wchar_t *wstr, size_t len) {
#ifdef _WIN32
    int res = MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, utf8, -1, wstr, (int)len);
    if (res > 0) {
        return VY_SUCCESS;
    } else {
        DWORD err = GetLastError();
        if (err == ERROR_INSUFFICIENT_BUFFER)
            return VY_INSUFFICIENT_BUFFER;
        else if (err == ERROR_NO_UNICODE_TRANSLATION)
            return VY_INVALID_UNICODE;
        else {
            vyReportError("CORE", "Unexpected error in vyUTF8ToWStr(): %u", err);
            return VY_UNKNOWN_ERROR;
        }
    }
#endif
}


VY_DLLEXPORT vy_result vyWStrToUTF8(const wchar_t *wstr, char *utf8, size_t len) {
#ifdef _WIN32
    int res = WideCharToMultiByte(CP_UTF8, WC_COMPOSITECHECK, wstr, -1, utf8, (int)len, NULL, NULL);
    if (res > 0) {
        return VY_SUCCESS;
    } else {
        DWORD err = GetLastError();
        if (err == ERROR_INSUFFICIENT_BUFFER)
            return VY_INSUFFICIENT_BUFFER;
        else if (err == ERROR_NO_UNICODE_TRANSLATION)
            return VY_INVALID_UNICODE;
        else {
            vyReportError("CORE", "Unexpected error in vyWStrToUTF8(): %u", err);
            return VY_UNKNOWN_ERROR;
        }
    }
#endif
}